import matplotlib.pyplot as plt
import numpy as np
from tqdm import tqdm_notebook

######################## For Autoencoders ##########################
def show_digit(original, fake, n, i, titles=["Sampled"], logs_dir=None, image_size=(28,28)):
    f, axes = plt.subplots(1, n, sharey=True)
    axes[0].imshow(original.reshape(image_size),cmap = plt.cm.gray)
    axes[0].set_title('Original ' + str(i))
    for j in range(1, n):
        axes[j].imshow(fake[j-1].reshape(image_size),cmap = plt.cm.gray)
        axes[j].set_title(str(i) + " " + titles[j-1])
    if logs_dir != None:
        plt.savefig(logs_dir + "sample_digit_" + str(i) + ".png")
    plt.show()
    plt.close('all')

def plot_images(logs_dir, images, image_size=(28,28), title="Misc"):
    len_ = int(np.sqrt(len(images)))
    fig = plt.figure()
    for x in range(len_):
        for y in range(len_):
            ax = fig.add_subplot(len_, len_, len_*y+x+1)
            ax.matshow(images[len_*y+x].reshape(image_size), cmap = plt.cm.gray)
            plt.xticks(np.array([]))
            plt.yticks(np.array([]))
    plt.savefig(logs_dir + title + ".png")
    plt.show()

def display_original_vs_reconstructed(logs_dir, decoded_imgs, original_imgs, n=10, _id=1, image_size=(28,28)):
    plt.figure(figsize=(20, 4))
    for i in range(1, n + 1):
        # Display original
        ax = plt.subplot(2, n, i)
        plt.imshow(original_imgs[i].reshape(image_size))
        plt.gray()
        ax.get_xaxis().set_visible(False)
        ax.get_yaxis().set_visible(False)

        # Display reconstruction
        ax = plt.subplot(2, n, i + n)
        plt.imshow(decoded_imgs[i].reshape(image_size))
        plt.gray()
        ax.get_xaxis().set_visible(False)
        ax.get_yaxis().set_visible(False)
    plt.savefig(logs_dir + "rec_digits_" + str(_id) + ".png")

def display_latent_space(logs_dir, model, encoded_imgs, n=10, _id=1, latent_size=(8, 18)):
    plt.figure(figsize=(20, 8))
    for i in range(1, n + 1):
        ax = plt.subplot(1, n, i)
        if model.ae_type == "DAE":
            plt.imshow(encoded_imgs[i].reshape(latent_size).T)
        else:
            plt.imshow(encoded_imgs[i].reshape((model.encoding_size[0], model.encoding_size[1] * model.encoding_size[2])).T)
        plt.gray()
        ax.get_xaxis().set_visible(False)
        ax.get_yaxis().set_visible(False)
    plt.savefig(logs_dir + "latent_space_" + str(_id) + ".png")

def display_loss_and_accuracy(logs_dir, history, _id=1):
    plt.figure()
    # Plot Accuracy
    plt.plot(history.history['accuracy'])
    plt.plot(history.history['val_accuracy'])
    plt.title('model accuracy')
    plt.ylabel('accuracy')
    plt.xlabel('epoch')
    plt.legend(['train', 'test'], loc='upper right')
    plt.savefig(logs_dir + "accuracy_" + str(_id) + ".png")
    plt.show()
    # Plot Loss
    plt.figure()
    plt.plot(history.history['loss'])
    plt.plot(history.history['val_loss'])
    plt.title('model loss')
    plt.ylabel('loss')
    plt.xlabel('epoch')
    plt.legend(['train', 'test'], loc='upper right')
    plt.savefig(logs_dir + "loss_" + str(_id) + ".png")
    plt.show()
    plt.close("all")

def predict_on_test(latent_test_1, num_mlps, bridge, subset):
    predicted_latent_test_2 = []

    for j in tqdm_notebook(range(len(latent_test_1))):
        latent_1 = np.array(latent_test_1[j]).reshape(1, -1)
#         latent_1 = latent_1.reshape(latent_1.shape[0], latent_1.shape[1], 1) # For LSTM

        ## Regress the hidden layer of the second RBM using the hiddens of the first RBM
        latent_2_predicted = []
        for k in range(int(num_mlps/subset)): # 
            model = bridge.trained_models[k]
            latent_2_node_predicted = model.predict(latent_1)
            latent_2_predicted.extend(latent_2_node_predicted)

        latent_2_predicted = np.array(latent_2_predicted)
        latent_2_predicted = latent_2_predicted.reshape(len(latent_2_predicted) * subset).astype(np.float32)
        predicted_latent_test_2.append(latent_2_predicted)

    predicted_latent_test_2 = np.array(predicted_latent_test_2)
    return predicted_latent_test_2
#######################################################################################